@@ -1,5 +1,5 @@
 #!/usr/bin/python3 -OO
-# Copyright 2007-2020 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2021 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -26,6 +26,7 @@ import logging
 import sys
 import threading
 import sqlite3
+from typing import Union, Dict, Optional, List
 
 import sabnzbd
 import sabnzbd.cfg
@@ -34,13 +35,13 @@ from sabnzbd.bpsmeter import this_week, this_month
 from sabnzbd.decorators import synchronized
 from sabnzbd.encoding import ubtou, utob
 from sabnzbd.misc import int_conv, caller_name, opts_to_pp
-from sabnzbd.filesystem import remove_file
+from sabnzbd.filesystem import remove_file, clip_path
 
 DB_LOCK = threading.RLock()
 
 
 def convert_search(search):
-    """ Convert classic wildcard to SQL wildcard """
+    """Convert classic wildcard to SQL wildcard"""
     if not search:
         # Default value
         search = ""
@@ -61,10 +62,10 @@ def convert_search(search):
 
 
 class HistoryDB:
-    """ Class to access the History database
-        Each class-instance will create an access channel that
-        can be used in one thread.
-        Each thread needs its own class-instance!
+    """Class to access the History database
+    Each class-instance will create an access channel that
+    can be used in one thread.
+    Each thread needs its own class-instance!
     """
 
     # These class attributes will be accessed directly because
@@ -74,17 +75,17 @@ class HistoryDB:
 
     @synchronized(DB_LOCK)
     def __init__(self):
-        """ Determine databse path and create connection """
+        """Determine databse path and create connection"""
         self.con = self.c = None
         if not HistoryDB.db_path:
             HistoryDB.db_path = os.path.join(sabnzbd.cfg.admin_dir.get_path(), DB_HISTORY_NAME)
         self.connect()
 
     def connect(self):
-        """ Create a connection to the database """
+        """Create a connection to the database"""
         create_table = not os.path.exists(HistoryDB.db_path)
         self.con = sqlite3.connect(HistoryDB.db_path)
-        self.con.row_factory = dict_factory
+        self.con.row_factory = sqlite3.Row
         self.c = self.con.cursor()
         if create_table:
             self.create_history_db()
@@ -98,7 +99,7 @@ class HistoryDB:
         self.execute("PRAGMA user_version;")
         try:
             version = self.c.fetchone()["user_version"]
-        except TypeError:
+        except IndexError:
             version = 0
         if version < 1:
             # Add any missing columns added since first DB version
@@ -116,7 +117,7 @@ class HistoryDB:
             )
 
     def execute(self, command, args=(), save=False):
-        """ Wrapper for executing SQL commands """
+        """Wrapper for executing SQL commands"""
         for tries in range(5, 0, -1):
             try:
                 if args and isinstance(args, tuple):
@@ -160,7 +161,7 @@ class HistoryDB:
             return False
 
     def create_history_db(self):
-        """ Create a new (empty) database file """
+        """Create a new (empty) database file"""
         self.execute(
             """
         CREATE TABLE "history" (
@@ -197,7 +198,7 @@ class HistoryDB:
         self.execute("PRAGMA user_version = 2;")
 
     def close(self):
-        """ Close database connection """
+        """Close database connection"""
         try:
             self.c.close()
             self.con.close()
@@ -206,7 +207,7 @@ class HistoryDB:
             logging.info("Traceback: ", exc_info=True)
 
     def remove_completed(self, search=None):
-        """ Remove all completed jobs from the database, optional with `search` pattern """
+        """Remove all completed jobs from the database, optional with `search` pattern"""
         search = convert_search(search)
         logging.info("Removing all completed jobs from history")
         return self.execute(
@@ -214,18 +215,18 @@ class HistoryDB:
         )
 
     def get_failed_paths(self, search=None):
-        """ Return list of all storage paths of failed jobs (may contain non-existing or empty paths) """
+        """Return list of all storage paths of failed jobs (may contain non-existing or empty paths)"""
         search = convert_search(search)
         fetch_ok = self.execute(
             """SELECT path FROM history WHERE name LIKE ? AND status = ?""", (search, Status.FAILED)
         )
         if fetch_ok:
-            return [item.get("path") for item in self.c.fetchall()]
+            return [item["path"] for item in self.c.fetchall()]
         else:
             return []
 
     def remove_failed(self, search=None):
-        """ Remove all failed jobs from the database, optional with `search` pattern """
+        """Remove all failed jobs from the database, optional with `search` pattern"""
         search = convert_search(search)
         logging.info("Removing all failed jobs from history")
         return self.execute(
@@ -233,7 +234,7 @@ class HistoryDB:
         )
 
     def remove_history(self, jobs=None):
-        """ Remove all jobs in the list `jobs`, empty list will remove all completed jobs """
+        """Remove all jobs in the list `jobs`, empty list will remove all completed jobs"""
         if jobs is None:
             self.remove_completed()
         else:
@@ -245,7 +246,7 @@ class HistoryDB:
                 logging.info("[%s] Removing job %s from history", caller_name(), job)
 
     def auto_history_purge(self):
-        """ Remove history items based on the configured history-retention """
+        """Remove history items based on the configured history-retention"""
         if sabnzbd.cfg.history_retention() == "0":
             return
 
@@ -270,27 +271,37 @@ class HistoryDB:
             if to_keep > 0:
                 logging.info("Removing all but last %s completed jobs from history", to_keep)
                 return self.execute(
-                    """DELETE FROM history WHERE status = ? AND id NOT IN ( SELECT id FROM history WHERE status = ? ORDER BY completed DESC LIMIT ? )""",
+                    """DELETE FROM history WHERE status = ? AND id NOT IN ( 
+                        SELECT id FROM history WHERE status = ? ORDER BY completed DESC LIMIT ? 
+                    )""",
                     (Status.COMPLETED, Status.COMPLETED, to_keep),
                     save=True,
                 )
 
-    def add_history_db(self, nzo, storage="", path="", postproc_time=0, script_output="", script_line=""):
-        """ Add a new job entry to the database """
-        t = build_history_info(nzo, storage, path, postproc_time, script_output, script_line, series_info=True)
+    def add_history_db(self, nzo, storage="", postproc_time=0, script_output="", script_line=""):
+        """Add a new job entry to the database"""
+        t = build_history_info(nzo, storage, postproc_time, script_output, script_line, series_info=True)
 
         self.execute(
             """INSERT INTO history (completed, name, nzb_name, category, pp, script, report,
             url, status, nzo_id, storage, path, script_log, script_line, download_time, postproc_time, stage_log,
-            downloaded, completeness, fail_message, url_info, bytes, series, md5sum, password)
-            VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)""",
+            downloaded, fail_message, url_info, bytes, series, md5sum, password)
+            VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)""",
             t,
             save=True,
         )
         logging.info("Added job %s to history", nzo.final_name)
 
-    def fetch_history(self, start=None, limit=None, search=None, failed_only=0, categories=None):
-        """ Return records for specified jobs """
+    def fetch_history(
+        self,
+        start: Optional[int] = None,
+        limit: Optional[int] = None,
+        search: Optional[str] = None,
+        failed_only: int = 0,
+        categories: Optional[List[str]] = None,
+        nzo_ids: Optional[List[str]] = None,
+    ):
+        """Return records for specified jobs"""
         command_args = [convert_search(search)]
 
         post = ""
@@ -300,18 +311,19 @@ class HistoryDB:
             post += " OR CATEGORY = ? " * (len(categories) - 1)
             post += ")"
             command_args.extend(categories)
+        if nzo_ids:
+            post += " AND (NZO_ID = ?"
+            post += " OR NZO_ID = ? " * (len(nzo_ids) - 1)
+            post += ")"
+            command_args.extend(nzo_ids)
         if failed_only:
             post += " AND STATUS = ?"
             command_args.append(Status.FAILED)
 
         cmd = "SELECT COUNT(*) FROM history WHERE name LIKE ?"
-        res = self.execute(cmd + post, tuple(command_args))
         total_items = -1
-        if res:
-            try:
-                total_items = self.c.fetchone().get("COUNT(*)")
-            except AttributeError:
-                pass
+        if self.execute(cmd + post, tuple(command_args)):
+            total_items = self.c.fetchone()["COUNT(*)"]
 
         if not start:
             start = 0
@@ -320,9 +332,7 @@ class HistoryDB:
 
         command_args.extend([start, limit])
         cmd = "SELECT * FROM history WHERE name LIKE ?"
-        fetch_ok = self.execute(cmd + post + " ORDER BY completed desc LIMIT ?, ?", tuple(command_args))
-
-        if fetch_ok:
+        if self.execute(cmd + post + " ORDER BY completed desc LIMIT ?, ?", tuple(command_args)):
             items = self.c.fetchall()
         else:
             items = []
@@ -336,138 +346,116 @@ class HistoryDB:
         return items, fetched_items, total_items
 
     def have_episode(self, series, season, episode):
-        """ Check whether History contains this series episode """
+        """Check whether History contains this series episode"""
         total = 0
         series = series.lower().replace(".", " ").replace("_", " ").replace("  ", " ")
         if series and season and episode:
             pattern = "%s/%s/%s" % (series, season, episode)
-            res = self.execute(
-                "select count(*) from History WHERE series = ? AND STATUS != ?", (pattern, Status.FAILED)
-            )
-            if res:
-                try:
-                    total = self.c.fetchone().get("count(*)")
-                except AttributeError:
-                    pass
+            if self.execute(
+                """SELECT COUNT(*) FROM History WHERE series = ? AND STATUS != ?""", (pattern, Status.FAILED)
+            ):
+                total = self.c.fetchone()["COUNT(*)"]
         return total > 0
 
     def have_name_or_md5sum(self, name, md5sum):
-        """ Check whether this name or md5sum is already in History """
+        """Check whether this name or md5sum is already in History"""
         total = 0
-        res = self.execute("select count(*) from History WHERE md5sum = ? AND STATUS != ?", (md5sum, Status.FAILED))
-        if res:
-            try:
-                total = self.c.fetchone().get("count(*)")
-            except AttributeError:
-                pass
+        if self.execute(
+            """SELECT COUNT(*) FROM History WHERE ( LOWER(name) = LOWER(?) OR md5sum = ? ) AND STATUS != ?""",
+            (name, md5sum, Status.FAILED),
+        ):
+            total = self.c.fetchone()["COUNT(*)"]
         return total > 0
 
     def get_history_size(self):
-        """ Returns the total size of the history and
-            amounts downloaded in the last month and week
+        """Returns the total size of the history and
+        amounts downloaded in the last month and week
         """
         # Total Size of the history
         total = 0
         if self.execute("""SELECT sum(bytes) FROM history"""):
-            try:
-                total = self.c.fetchone().get("sum(bytes)")
-            except AttributeError:
-                pass
+            total = self.c.fetchone()["sum(bytes)"]
 
         # Amount downloaded this month
-        # r = time.gmtime(time.time())
-        # month_timest = int(time.mktime((r.tm_year, r.tm_mon, 0, 0, 0, 1, r.tm_wday, r.tm_yday, r.tm_isdst)))
         month_timest = int(this_month(time.time()))
 
         month = 0
         if self.execute("""SELECT sum(bytes) FROM history WHERE completed > ?""", (month_timest,)):
-            try:
-                month = self.c.fetchone().get("sum(bytes)")
-            except AttributeError:
-                pass
+            month = self.c.fetchone()["sum(bytes)"]
 
         # Amount downloaded this week
         week_timest = int(this_week(time.time()))
 
         week = 0
         if self.execute("""SELECT sum(bytes) FROM history WHERE completed > ?""", (week_timest,)):
-            try:
-                week = self.c.fetchone().get("sum(bytes)")
-            except AttributeError:
-                pass
+            week = self.c.fetchone()["sum(bytes)"]
 
         return total, month, week
 
     def get_script_log(self, nzo_id):
-        """ Return decompressed log file """
+        """Return decompressed log file"""
         data = ""
         t = (nzo_id,)
-        if self.execute("SELECT script_log FROM history WHERE nzo_id = ?", t):
+        if self.execute("""SELECT script_log FROM history WHERE nzo_id = ?""", t):
             try:
-                data = ubtou(zlib.decompress(self.c.fetchone().get("script_log")))
+                data = ubtou(zlib.decompress(self.c.fetchone()["script_log"]))
             except:
                 pass
         return data
 
     def get_name(self, nzo_id):
-        """ Return name of the job `nzo_id` """
+        """Return name of the job `nzo_id`"""
         t = (nzo_id,)
         name = ""
-        if self.execute("SELECT name FROM history WHERE nzo_id = ?", t):
+        if self.execute("""SELECT name FROM history WHERE nzo_id = ?""", t):
             try:
-                name = self.c.fetchone().get("name")
-            except AttributeError:
+                name = self.c.fetchone()["name"]
+            except TypeError:
+                # No records found
                 pass
         return name
 
-    def get_path(self, nzo_id):
-        """ Return the `incomplete` path of the job `nzo_id` if it is still there """
+    def get_path(self, nzo_id: str):
+        """Return the `incomplete` path of the job `nzo_id` if it is still there"""
         t = (nzo_id,)
         path = ""
-        if self.execute("SELECT path FROM history WHERE nzo_id = ?", t):
+        if self.execute("""SELECT path FROM history WHERE nzo_id = ?""", t):
             try:
-                path = self.c.fetchone().get("path")
-            except AttributeError:
+                path = self.c.fetchone()["path"]
+            except TypeError:
+                # No records found
                 pass
         if os.path.exists(path):
             return path
         return None
 
     def get_other(self, nzo_id):
-        """ Return additional data for job `nzo_id` """
+        """Return additional data for job `nzo_id`"""
         t = (nzo_id,)
-        if self.execute("SELECT * FROM history WHERE nzo_id = ?", t):
+        if self.execute("""SELECT * FROM history WHERE nzo_id = ?""", t):
             try:
-                items = self.c.fetchone()
-                dtype = items.get("report")
-                url = items.get("url")
-                pp = items.get("pp")
-                script = items.get("script")
-                cat = items.get("category")
-                return dtype, url, pp, script, cat
-            except (AttributeError, IndexError):
+                item = self.c.fetchone()
+                return item["report"], item["url"], item["pp"], item["script"], item["category"]
+            except TypeError:
+                # No records found
                 pass
         return "", "", "", "", ""
 
+    def __enter__(self):
+        """For context manager support"""
+        return self
 
-def dict_factory(cursor, row):
-    """ Return a dictionary for the current database position """
-    d = {}
-    for idx, col in enumerate(cursor.description):
-        d[col[0]] = row[idx]
-    return d
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        """For context manager support, ignore any exception"""
+        self.close()
 
 
 _PP_LOOKUP = {0: "", 1: "R", 2: "U", 3: "D"}
 
 
-def build_history_info(
-    nzo, storage="", downpath="", postproc_time=0, script_output="", script_line="", series_info=False
-):
-    """ Collects all the information needed for the database """
+def build_history_info(nzo, workdir_complete="", postproc_time=0, script_output="", script_line="", series_info=False):
+    """Collects all the information needed for the database"""
     completed = int(time.time())
-    if not downpath:
-        downpath = nzo.downpath
     pp = _PP_LOOKUP.get(opts_to_pp(*nzo.repair_opts), "X")
 
     if script_output:
@@ -475,7 +463,6 @@ def build_history_info(
         script_output = sqlite3.Binary(zlib.compress(utob(script_output)))
 
     download_time = nzo.nzo_info.get("download_time", 0)
-    completeness = 0
     url_info = nzo.nzo_info.get("details", "") or nzo.nzo_info.get("more_info", "")
 
     # Get the dictionary containing the stages and their unpack process
@@ -507,15 +494,14 @@ def build_history_info(
         nzo.url,
         nzo.status,
         nzo.nzo_id,
-        storage,
-        downpath,
+        clip_path(workdir_complete),
+        clip_path(nzo.download_path),
         script_output,
         script_line,
         download_time,
         postproc_time,
         stage_log,
         nzo.bytes_downloaded,
-        completeness,
         nzo.fail_msg,
         url_info,
         nzo.bytes_downloaded,
@@ -525,44 +511,42 @@ def build_history_info(
     )
 
 
-def unpack_history_info(item):
-    """ Expands the single line stage_log from the DB
-        into a python dictionary for use in the history display
+def unpack_history_info(item: Union[Dict, sqlite3.Row]):
+    """Expands the single line stage_log from the DB
+    into a python dictionary for use in the history display
     """
+    # Convert result to dictionary
+    if isinstance(item, sqlite3.Row):
+        item = dict(item)
+
     # Stage Name is separated by ::: stage lines by ; and stages by \r\n
     lst = item["stage_log"]
     if lst:
+        parsed_stage_log = []
         try:
-            lines = lst.split("\r\n")
+            all_stages_lines = lst.split("\r\n")
         except:
-            logging.error(T("Invalid stage logging in history for %s") + " (\\r\\n)", item["name"])
+            logging.error(T("Invalid stage logging in history for %s"), item["name"])
             logging.debug("Lines: %s", lst)
-            lines = []
-        lst = [None for x in STAGES]
-        for line in lines:
-            stage = {}
+            all_stages_lines = []
+
+        for stage_lines in all_stages_lines:
             try:
-                key, logs = line.split(":::")
+                key, logs = stage_lines.split(":::")
             except:
-                logging.debug('Missing key:::logs "%s"', line)
-                key = line
-                logs = ""
-            stage["name"] = key
-            stage["actions"] = []
+                logging.info('Missing key:::logs "%s"', stage_lines)
+                continue
+            stage = {"name": key, "actions": []}
             try:
-                logs = logs.split(";")
+                stage["actions"] = logs.split(";")
             except:
-                logging.error(T("Invalid stage logging in history for %s") + " (;)", item["name"])
+                logging.error(T("Invalid stage logging in history for %s"), item["name"])
                 logging.debug("Logs: %s", logs)
-                logs = []
-            for log in logs:
-                stage["actions"].append(log)
-            try:
-                lst[STAGES[key]] = stage
-            except KeyError:
-                lst.append(stage)
-        # Remove unused stages
-        item["stage_log"] = [x for x in lst if x is not None]
+            parsed_stage_log.append(stage)
+
+        # Sort it so it is more logical
+        parsed_stage_log.sort(key=lambda stage_log: STAGES.get(stage_log["name"], 100))
+        item["stage_log"] = parsed_stage_log
 
     if item["script_log"]:
         item["script_log"] = ""
@@ -574,6 +558,5 @@ def unpack_history_info(item):
 
 def midnight_history_purge():
     logging.info("Scheduled history purge")
-    history_db = HistoryDB()
-    history_db.auto_history_purge()
-    history_db.close()
+    with HistoryDB() as history_db:
+        history_db.auto_history_purge()
