@@ -23,14 +23,17 @@ import logging
 import os
 import re
 import struct
-from typing import Dict, Optional, Tuple
+from typing import Dict, Optional, Tuple, BinaryIO
 
+from sabnzbd.constants import MEBI
 from sabnzbd.encoding import correct_unknown_encoding
 
 PROBABLY_PAR2_RE = re.compile(r"(.*)\.vol(\d*)[+\-](\d*)\.par2", re.I)
+SCAN_LIMIT = 10 * MEBI
 PAR_PKT_ID = b"PAR2\x00PKT"
+PAR_MAIN_ID = b"PAR 2.0\x00Main\x00\x00\x00\x00"
 PAR_FILE_ID = b"PAR 2.0\x00FileDesc"
-PAR_CREATOR_ID = b"PAR 2.0\x00Creator"
+PAR_CREATOR_ID = b"PAR 2.0\x00Creator\x00"
 PAR_RECOVERY_ID = b"RecvSlic"
 
 
@@ -91,22 +94,34 @@ def parse_par2_file(fname: str, md5of16k: Dict[bytes, str]) -> Dict[str, bytes]:
     For a full description of the par2 specification, visit:
     http://parchive.sourceforge.net/docs/specifications/parity-volume-spec/article-spec.html
     """
+    total_size = os.path.getsize(fname)
     table = {}
     duplicates16k = []
+    total_nr_files = None
 
     try:
         with open(fname, "rb") as f:
             header = f.read(8)
             while header:
-                name, filehash, hash16k = parse_par2_file_packet(f, header)
-                if name:
-                    table[name] = filehash
-                    if hash16k not in md5of16k:
-                        md5of16k[hash16k] = name
-                    elif md5of16k[hash16k] != name:
-                        # Not unique and not already linked to this file
-                        # Remove to avoid false-renames
-                        duplicates16k.append(hash16k)
+                if header == PAR_PKT_ID:
+                    name, filehash, hash16k, nr_files = parse_par2_packet(f)
+                    if name:
+                        table[name] = filehash
+                        if hash16k not in md5of16k:
+                            md5of16k[hash16k] = name
+                        elif md5of16k[hash16k] != name:
+                            # Not unique and not already linked to this file
+                            # Remove to avoid false-renames
+                            duplicates16k.append(hash16k)
+
+                    # Store the number of files for later
+                    if nr_files:
+                        total_nr_files = nr_files
+
+                    # On large files, we stop after seeing all the listings
+                    # On smaller files, we scan them fully to get the par2-creator
+                    if total_size > SCAN_LIMIT and len(table) == total_nr_files:
+                        break
 
                 header = f.read(8)
 
@@ -129,13 +144,18 @@ def parse_par2_file(fname: str, md5of16k: Dict[bytes, str]) -> Dict[str, bytes]:
     return table
 
 
-def parse_par2_file_packet(f, header) -> Tuple[Optional[str], Optional[bytes], Optional[bytes]]:
-    """ Look up and analyze a FileDesc package """
+def parse_par2_packet(f: BinaryIO) -> Tuple[Optional[str], Optional[bytes], Optional[bytes], Optional[int]]:
+    """Look up and analyze a PAR2 packet"""
 
-    nothing = None, None, None
+    filename, filehash, hash16k, nr_files = nothing = None, None, None, None
 
-    if header != PAR_PKT_ID:
-        return nothing
+    # All packages start with a header before the body
+    # 8	  : PAR2\x00PKT
+    # 8	  : Length of the entire packet. Must be multiple of 4. (NB: Includes length of header.)
+    # 16  : MD5 Hash of packet. Calculation starts at first byte of Recovery Set ID and ends at last byte of body.
+    # 16  : Recovery Set ID.
+    # 16  : Type of packet.
+    # ?*4 : Body of Packet. Must be a multiple of 4 bytes.
 
     # Length must be multiple of 4 and at least 20
     pack_len = struct.unpack("<Q", f.read(8))[0]
@@ -146,31 +166,37 @@ def parse_par2_file_packet(f, header) -> Tuple[Optional[str], Optional[bytes], O
     md5sum = f.read(16)
 
     # Read and check the data
+    # Subtract 32 because we already read these bytes of the header
     data = f.read(pack_len - 32)
     md5 = hashlib.md5()
     md5.update(data)
     if md5sum != md5.digest():
         return nothing
 
-    # The FileDesc packet looks like:
-    # 16 : "PAR 2.0\0FileDesc"
-    # 16 : FileId
-    # 16 : Hash for full file **
-    # 16 : Hash for first 16K
-    #  8 : File length
-    # xx : Name (multiple of 4, padded with \0 if needed) **
+    # See if it's any of the packages we care about
+    par2_packet_type = data[16:32]
 
-    # See if it's the right packet and get name + hash
-    for offset in range(0, pack_len, 8):
-        if data[offset : offset + 16] == PAR_FILE_ID:
-            filehash = data[offset + 32 : offset + 48]
-            hash16k = data[offset + 48 : offset + 64]
-            filename = correct_unknown_encoding(data[offset + 72 :].strip(b"\0"))
-            return filename, filehash, hash16k
-        elif data[offset : offset + 15] == PAR_CREATOR_ID:
-            # From here until the end is the creator-text
-            # Useful in case of bugs in the par2-creating software
-            par2creator = data[offset + 16 :].strip(b"\0")  # Remove any trailing \0
-            logging.debug("Par2-creator of %s is: %s", os.path.basename(f.name), correct_unknown_encoding(par2creator))
+    if par2_packet_type == PAR_FILE_ID:
+        # The FileDesc packet looks like:
+        # 16 : "PAR 2.0\0FileDesc"
+        # 16 : FileId
+        # 16 : Hash for full file
+        # 16 : Hash for first 16K
+        #  8 : File length
+        # xx : Name (multiple of 4, padded with \0 if needed)
+        filehash = data[48:64]
+        hash16k = data[64:80]
+        filename = correct_unknown_encoding(data[88:].strip(b"\0"))
+    elif par2_packet_type == PAR_CREATOR_ID:
+        # From here until the end is the creator-text
+        # Useful in case of bugs in the par2-creating software
+        par2creator = data[32:].strip(b"\0")  # Remove any trailing \0
+        logging.debug("Par2-creator of %s is: %s", os.path.basename(f.name), correct_unknown_encoding(par2creator))
+    elif par2_packet_type == PAR_MAIN_ID:
+        # The Main packet looks like:
+        # 16 : "PAR 2.0\0Main"
+        # 8  : Slice size
+        # 4  : Number of files in the recovery set
+        nr_files = struct.unpack("<I", data[40:44])[0]
 
-    return nothing
+    return filename, filehash, hash16k, nr_files
