@@ -1,4 +1,4 @@
-#!/usr/bin/python -OO
+#!/usr/bin/python3 -OO
 # Copyright 2008-2012 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
@@ -19,30 +19,33 @@
 sabnzbd.rating - Rating support functions
 """
 
-import httplib
-import urllib
-import urlparse
+import http.client
+import urllib.parse
 import time
 import logging
 import copy
-import socket
-import Queue
+import queue
 import collections
 from threading import RLock, Thread
+
 import sabnzbd
+from sabnzbd.constants import RATING_FILE_NAME
 from sabnzbd.decorators import synchronized
 import sabnzbd.cfg as cfg
 
 # A queue which ignores duplicates but maintains ordering
-class OrderedSetQueue(Queue.Queue):
+class OrderedSetQueue(queue.Queue):
     def _init(self, maxsize):
         self.maxsize = maxsize
         self.queue = collections.OrderedDict()
+
     def _put(self, item):
         self.queue[item] = None
+
     def _get(self):
         return self.queue.popitem()[0]
 
+
 _RATING_URL = "/releaseRatings/releaseRatings.php"
 RATING_LOCK = RLock()
 
@@ -61,8 +64,7 @@ def _reset_warn():
     _g_warnings = 0
 
 
-class NzbRating(object):
-
+class NzbRating:
     def __init__(self):
         self.avg_video = 0
         self.avg_video_cnt = 0
@@ -76,20 +78,14 @@ class NzbRating(object):
         self.user_flag = {}
         self.auto_flag = {}
         self.changed = 0
-
-
-class NzbRatingV2(NzbRating):
-
-    def __init__(self):
-        super(NzbRatingV2, self).__init__()
         self.avg_spam_cnt = 0
         self.avg_spam_confirm = False
         self.avg_encrypted_cnt = 0
         self.avg_encrypted_confirm = False
 
-    def to_v2(self, rating):
-        self.__dict__.update(rating.__dict__)
-        return self
+
+# TODO: Can be removed in version 3.3.0, needed for backwards compatibility
+NzbRatingV2 = NzbRating
 
 
 class Rating(Thread):
@@ -111,31 +107,20 @@ class Rating(Thread):
     CHANGED_USER_FLAG = 0x08
     CHANGED_AUTO_FLAG = 0x10
 
-    do = None
-
     def __init__(self):
-        Rating.do = self
         self.shutdown = False
         self.queue = OrderedSetQueue()
+        self.version = Rating.VERSION
+        self.ratings = {}
+        self.nzo_indexer_map = {}
         try:
-            self.version, self.ratings, self.nzo_indexer_map = sabnzbd.load_admin("Rating.sab",
-                                                                                  silent=not cfg.rating_enable())
-            if self.version == 1:
-                ratings = {}
-                for k, v in self.ratings.iteritems():
-                    ratings[k] = NzbRatingV2().to_v2(v)
-                self.ratings = ratings
-                self.version = 2
-            if self.version != Rating.VERSION:
-                raise Exception()
+            rating_data = sabnzbd.load_admin(RATING_FILE_NAME)
+            if rating_data:
+                self.version, self.ratings, self.nzo_indexer_map = rating_data
         except:
-            self.version = Rating.VERSION
-            self.ratings = {}
-            self.nzo_indexer_map = {}
-        Thread.__init__(self)
-        if not sabnzbd.HAVE_SSL:
-            logging.warning(T('Ratings server requires secure connection'))
-            self.stop()
+            logging.info("Corrupt %s file, discarding", RATING_FILE_NAME)
+            logging.info("Traceback: ", exc_info=True)
+        super().__init__()
 
     def stop(self):
         self.shutdown = True
@@ -157,43 +142,54 @@ class Rating(Thread):
                     self.queue.put(indexer_id)
             except:
                 pass
-        logging.debug('Stopping ratings')
+        logging.debug("Stopping ratings")
 
     @synchronized(RATING_LOCK)
     def save(self):
-        if self.ratings and self.nzo_indexer_map:
-            sabnzbd.save_admin((self.version, self.ratings, self.nzo_indexer_map), "Rating.sab")
+        sabnzbd.save_admin((self.version, self.ratings, self.nzo_indexer_map), RATING_FILE_NAME)
 
     # The same file may be uploaded multiple times creating a new nzo_id each time
     @synchronized(RATING_LOCK)
     def add_rating(self, indexer_id, nzo_id, fields):
         if indexer_id and nzo_id:
-            logging.debug('Add rating (%s, %s: %s, %s, %s, %s)', indexer_id, nzo_id, fields['video'], fields['audio'], fields['voteup'], fields['votedown'])
+            logging.debug(
+                "Add rating (%s, %s: %s, %s, %s, %s)",
+                indexer_id,
+                nzo_id,
+                fields["video"],
+                fields["audio"],
+                fields["voteup"],
+                fields["votedown"],
+            )
             try:
-                rating = self.ratings.get(indexer_id, NzbRatingV2())
-                if fields['video'] and fields['videocnt']:
-                    rating.avg_video = int(float(fields['video']))
-                    rating.avg_video_cnt = int(float(fields['videocnt']))
-                if fields['audio'] and fields['audiocnt']:
-                    rating.avg_audio = int(float(fields['audio']))
-                    rating.avg_audio_cnt = int(float(fields['audiocnt']))
-                if fields['voteup']:
-                    rating.avg_vote_up = int(float(fields['voteup']))
-                if fields['votedown']:
-                    rating.avg_vote_down = int(float(fields['votedown']))
-                if fields['spam']:
-                    rating.avg_spam_cnt = int(float(fields['spam']))
-                if fields['confirmed-spam']:
-                    rating.avg_spam_confirm = (fields['confirmed-spam'].lower() == 'yes')
-                if fields['passworded']:
-                    rating.avg_encrypted_cnt = int(float(fields['passworded']))
-                if fields['confirmed-passworded']:
-                    rating.avg_encrypted_confirm = (fields['confirmed-passworded'].lower() == 'yes')
+                rating = self.ratings.get(indexer_id, NzbRating())
+                if fields["video"] and fields["videocnt"]:
+                    rating.avg_video = int(float(fields["video"]))
+                    rating.avg_video_cnt = int(float(fields["videocnt"]))
+                if fields["audio"] and fields["audiocnt"]:
+                    rating.avg_audio = int(float(fields["audio"]))
+                    rating.avg_audio_cnt = int(float(fields["audiocnt"]))
+                if fields["voteup"]:
+                    rating.avg_vote_up = int(float(fields["voteup"]))
+                if fields["votedown"]:
+                    rating.avg_vote_down = int(float(fields["votedown"]))
+                if fields["spam"]:
+                    rating.avg_spam_cnt = int(float(fields["spam"]))
+                if fields["confirmed-spam"]:
+                    rating.avg_spam_confirm = fields["confirmed-spam"].lower() == "yes"
+                if fields["passworded"]:
+                    rating.avg_encrypted_cnt = int(float(fields["passworded"]))
+                if fields["confirmed-passworded"]:
+                    rating.avg_encrypted_confirm = fields["confirmed-passworded"].lower() == "yes"
                 # Indexers can supply a full URL or just a host
-                if fields['host']:
-                    rating.host = fields['host'][0] if fields['host'] and isinstance(fields['host'], list) else fields['host']
-                if fields['url']:
-                    rating.host = fields['url'][0] if fields['url'] and isinstance(fields['url'], list) else fields['url']
+                if fields["host"]:
+                    rating.host = (
+                        fields["host"][0] if fields["host"] and isinstance(fields["host"], list) else fields["host"]
+                    )
+                if fields["url"]:
+                    rating.host = (
+                        fields["url"][0] if fields["url"] and isinstance(fields["url"], list) else fields["url"]
+                    )
                 self.ratings[indexer_id] = rating
                 self.nzo_indexer_map[nzo_id] = indexer_id
             except:
@@ -201,22 +197,26 @@ class Rating(Thread):
 
     @synchronized(RATING_LOCK)
     def update_user_rating(self, nzo_id, video, audio, vote, flag, flag_detail=None):
-        logging.debug('Updating user rating (%s: %s, %s, %s, %s)', nzo_id, video, audio, vote, flag)
+        logging.debug("Updating user rating (%s: %s, %s, %s, %s)", nzo_id, video, audio, vote, flag)
         if nzo_id not in self.nzo_indexer_map:
-            logging.warning(T('Indexer id (%s) not found for ratings file'), nzo_id)
+            logging.warning(T("Indexer id (%s) not found for ratings file"), nzo_id)
             return
         indexer_id = self.nzo_indexer_map[nzo_id]
         rating = self.ratings[indexer_id]
         if video:
             rating.user_video = int(video)
-            rating.avg_video = int((rating.avg_video_cnt * rating.avg_video + rating.user_video) / (rating.avg_video_cnt + 1))
+            rating.avg_video = int(
+                (rating.avg_video_cnt * rating.avg_video + rating.user_video) / (rating.avg_video_cnt + 1)
+            )
             rating.changed = rating.changed | Rating.CHANGED_USER_VIDEO
         if audio:
             rating.user_audio = int(audio)
-            rating.avg_audio = int((rating.avg_audio_cnt * rating.avg_audio + rating.user_audio) / (rating.avg_audio_cnt + 1))
+            rating.avg_audio = int(
+                (rating.avg_audio_cnt * rating.avg_audio + rating.user_audio) / (rating.avg_audio_cnt + 1)
+            )
             rating.changed = rating.changed | Rating.CHANGED_USER_AUDIO
         if flag:
-            rating.user_flag = {'val': int(flag), 'detail': flag_detail}
+            rating.user_flag = {"val": int(flag), "detail": flag_detail}
             rating.changed = rating.changed | Rating.CHANGED_USER_FLAG
         if vote:
             rating.changed = rating.changed | Rating.CHANGED_USER_VOTE
@@ -236,12 +236,12 @@ class Rating(Thread):
 
     @synchronized(RATING_LOCK)
     def update_auto_flag(self, nzo_id, flag, flag_detail=None):
-        if not flag or not cfg.rating_enable() or not cfg.rating_feedback() or (nzo_id not in self.nzo_indexer_map):
+        if not flag or not cfg.rating_enable() or (nzo_id not in self.nzo_indexer_map):
             return
-        logging.debug('Updating auto flag (%s: %s)', nzo_id, flag)
+        logging.debug("Updating auto flag (%s: %s)", nzo_id, flag)
         indexer_id = self.nzo_indexer_map[nzo_id]
         rating = self.ratings[indexer_id]
-        rating.auto_flag = {'val': int(flag), 'detail': flag_detail}
+        rating.auto_flag = {"val": int(flag), "detail": flag_detail}
         rating.changed = rating.changed | Rating.CHANGED_AUTO_FLAG
         self.queue.put(indexer_id)
 
@@ -257,68 +257,86 @@ class Rating(Thread):
 
     def _flag_request(self, val, flag_detail, auto):
         if val == Rating.FLAG_SPAM:
-            return {'m': 'rs', 'auto': auto}
+            return {"m": "rs", "auto": auto}
         if val == Rating.FLAG_ENCRYPTED:
-            return {'m': 'rp', 'auto': auto}
+            return {"m": "rp", "auto": auto}
         if val == Rating.FLAG_EXPIRED:
-            expired_host = flag_detail if flag_detail and len(flag_detail) > 0 else 'Other'
-            return {'m': 'rpr', 'pr': expired_host, 'auto': auto}
+            expired_host = flag_detail if flag_detail and len(flag_detail) > 0 else "Other"
+            return {"m": "rpr", "pr": expired_host, "auto": auto}
         if (val == Rating.FLAG_OTHER) and flag_detail and len(flag_detail) > 0:
-            return {'m': 'o', 'r': flag_detail}
+            return {"m": "o", "r": flag_detail}
         if (val == Rating.FLAG_COMMENT) and flag_detail and len(flag_detail) > 0:
-            return {'m': 'rc', 'r': flag_detail}
+            return {"m": "rc", "r": flag_detail}
 
     def _send_rating(self, indexer_id):
-        logging.debug('Updating indexer rating (%s)', indexer_id)
+        logging.debug("Updating indexer rating (%s)", indexer_id)
 
         api_key = cfg.rating_api_key()
         rating_host = cfg.rating_host()
         rating_url = _RATING_URL
-        if not api_key:
-            return True
 
         requests = []
-        _headers = {'User-agent': 'SABnzbd+/%s' % sabnzbd.version.__version__, 'Content-type': 'application/x-www-form-urlencoded'}
-        rating = self._get_rating_by_indexer(indexer_id)  # Requesting info here ensures always have latest information even on retry
-        if hasattr(rating, 'host') and rating.host:
-            host_parsed = urlparse.urlparse(rating.host)
+        _headers = {
+            "User-agent": "SABnzbd/%s" % sabnzbd.__version__,
+            "Content-type": "application/x-www-form-urlencoded",
+        }
+        rating = self._get_rating_by_indexer(
+            indexer_id
+        )  # Requesting info here ensures always have latest information even on retry
+        if hasattr(rating, "host") and rating.host:
+            host_parsed = urllib.parse.urlparse(rating.host)
             rating_host = host_parsed.netloc
             # Is it an URL or just a HOST?
-            if host_parsed.path and host_parsed.path != '/':
-                rating_url = host_parsed.path + '?' + host_parsed.query if host_parsed.query else host_parsed.path
+            if host_parsed.path and host_parsed.path != "/":
+                rating_url = host_parsed.path + "?" + host_parsed.query if host_parsed.query else host_parsed.path
+
         if not rating_host:
+            _warn("%s: %s" % (T("Cannot send, missing required data"), T("Server address")))
             return True
+
+        if not api_key:
+            _warn(
+                "%s [%s]: %s - %s"
+                % (
+                    T("Cannot send, missing required data"),
+                    rating_host,
+                    T("API Key"),
+                    T("This key provides identity to indexer. Check your profile on the indexer's website."),
+                )
+            )
+            return True
+
         if rating.changed & Rating.CHANGED_USER_VIDEO:
-            requests.append({'m': 'r', 'r': 'videoQuality', 'rn': rating.user_video})
+            requests.append({"m": "r", "r": "videoQuality", "rn": rating.user_video})
         if rating.changed & Rating.CHANGED_USER_AUDIO:
-            requests.append({'m': 'r', 'r': 'audioQuality', 'rn': rating.user_audio})
+            requests.append({"m": "r", "r": "audioQuality", "rn": rating.user_audio})
         if rating.changed & Rating.CHANGED_USER_VOTE:
-            up_down = 'up' if rating.user_vote == Rating.VOTE_UP else 'down'
-            requests.append({'m': 'v', 'v': up_down, 'r': 'overall'})
+            up_down = "up" if rating.user_vote == Rating.VOTE_UP else "down"
+            requests.append({"m": "v", "v": up_down, "r": "overall"})
         if rating.changed & Rating.CHANGED_USER_FLAG:
-            requests.append(self._flag_request(rating.user_flag.get('val'), rating.user_flag.get('detail'), 0))
+            requests.append(self._flag_request(rating.user_flag.get("val"), rating.user_flag.get("detail"), 0))
         if rating.changed & Rating.CHANGED_AUTO_FLAG:
-            requests.append(self._flag_request(rating.auto_flag.get('val'), rating.auto_flag.get('detail'), 1))
+            requests.append(self._flag_request(rating.auto_flag.get("val"), rating.auto_flag.get("detail"), 1))
 
         try:
-            conn = httplib.HTTPSConnection(rating_host)
-            for request in filter(lambda r: r is not None, requests):
+            conn = http.client.HTTPSConnection(rating_host)
+            for request in [r for r in requests if r is not None]:
                 if api_key:
-                    request['apikey'] = api_key
-                request['i'] = indexer_id
-                conn.request('POST', rating_url, urllib.urlencode(request), headers=_headers)
+                    request["apikey"] = api_key
+                request["i"] = indexer_id
+                conn.request("POST", rating_url, urllib.parse.urlencode(request), headers=_headers)
 
                 response = conn.getresponse()
                 response.read()
-                if response.status == httplib.UNAUTHORIZED:
-                    _warn('Ratings server unauthorized user')
+                if response.status == http.client.UNAUTHORIZED:
+                    _warn("Ratings server unauthorized user")
                     return False
-                elif response.status != httplib.OK:
-                    _warn('Ratings server failed to process request (%s, %s)' % (response.status, response.reason))
+                elif response.status != http.client.OK:
+                    _warn("Ratings server failed to process request (%s, %s)" % (response.status, response.reason))
                     return False
             self.ratings[indexer_id].changed = self.ratings[indexer_id].changed & ~rating.changed
             _reset_warn()
             return True
         except:
-            _warn('Problem accessing ratings server: %s' % rating_host)
+            _warn("Problem accessing ratings server: %s" % rating_host)
             return False
